@@ -29,8 +29,8 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: chsh.c,v 1.30 2005/07/24 15:22:45 kloczek Exp $")
+#ident "$Id: chsh.c,v 1.40 2006/05/07 17:44:39 kloczek Exp $"
+
 #include <fcntl.h>
 #include <pwd.h>
 #include <signal.h>
@@ -56,7 +56,9 @@ RCSID (PKG_VER "$Id: chsh.c,v 1.30 2005/07/24 15:22:45 kloczek Exp $")
 #ifndef SHELLS_FILE
 #define SHELLS_FILE "/etc/shells"
 #endif
-/* global variables */
+/*
+ * Global variables
+ */
 static char *Prog;		/* Program name */
 static int amroot;		/* Real UID is root */
 static char loginsh[BUFSIZ];	/* Name of new login shell */
@@ -66,12 +68,12 @@ static char loginsh[BUFSIZ];	/* Name of new login shell */
 /* local function prototypes */
 static void usage (void);
 static void new_fields (void);
+static int check_shell (const char *);
 static int restricted_shell (const char *);
 
 /*
  * usage - print command line syntax and exit
  */
-
 static void usage (void)
 {
 	fprintf (stderr, _("Usage: %s [-s shell] [name]\n"), Prog);
@@ -84,7 +86,6 @@ static void usage (void)
  * prompt the user for the login shell and change it according to the
  * response, or leave it alone if nothing was entered.
  */
-
 static void new_fields (void)
 {
 	printf (_("Enter the new value, or press ENTER for the default\n"));
@@ -97,7 +98,6 @@ static void new_fields (void)
  * If the first letter of the filename is 'r' or 'R', the shell is
  * considered to be restricted.
  */
-
 static int restricted_shell (const char *sh)
 {
 	/*
@@ -108,6 +108,58 @@ static int restricted_shell (const char *sh)
 	return !check_shell (sh);
 }
 
+/*
+ * check_shell - see if the user's login shell is listed in /etc/shells
+ *
+ * The /etc/shells file is read for valid names of login shells.  If the
+ * /etc/shells file does not exist the user cannot set any shell unless
+ * they are root.
+ *
+ * If getusershell() is available (Linux, *BSD, possibly others), use it
+ * instead of re-implementing it.
+ */
+static int check_shell (const char *sh)
+{
+	char *cp;
+	int found = 0;
+
+#ifndef HAVE_GETUSERSHELL
+	char buf[BUFSIZ];
+	FILE *fp;
+#endif
+
+#ifdef HAVE_GETUSERSHELL
+	setusershell ();
+	while ((cp = getusershell ())) {
+		if (*cp == '#')
+			continue;
+
+		if (strcmp (cp, sh) == 0) {
+			found = 1;
+			break;
+		}
+	}
+	endusershell ();
+#else
+	if ((fp = fopen (SHELLS_FILE, "r")) == (FILE *) 0)
+		return 0;
+
+	while (fgets (buf, sizeof (buf), fp)) {
+		if ((cp = strrchr (buf, '\n')))
+			*cp = '\0';
+
+		if (buf[0] == '#')
+			continue;
+
+		if (strcmp (buf, sh) == 0) {
+			found = 1;
+			break;
+		}
+	}
+	fclose (fp);
+#endif
+	return found;
+}
 
 /*
  * chsh - this command controls changes to the user's shell
@@ -115,7 +167,6 @@ static int restricted_shell (const char *sh)
  *	The only supported option is -s which permits the the login shell to
  *	be set from the command line.
  */
-
 int main (int argc, char **argv)
 {
 	char *user;		/* User name                         */
@@ -124,6 +175,12 @@ int main (int argc, char **argv)
 	const struct passwd *pw;	/* Password entry from /etc/passwd   */
 	struct passwd pwent;	/* New password entry                */
 
+#ifdef USE_PAM
+	pam_handle_t *pamh = NULL;
+	struct passwd *pampw;
+	int retval;
+#endif
+
 	sanitize_env ();
 
 	setlocale (LC_ALL, "");
@@ -133,14 +190,12 @@ int main (int argc, char **argv)
 	/*
 	 * This command behaves different for root and non-root users.
 	 */
-
 	amroot = getuid () == 0;
 
 	/*
 	 * Get the program name. The program name is used as a prefix to
 	 * most error messages.
 	 */
-
 	Prog = Basename (argv[0]);
 
 	OPENLOG ("chsh");
@@ -165,7 +220,6 @@ int main (int argc, char **argv)
 	 * There should be only one remaining argument at most and it should
 	 * be the user's name.
 	 */
-
 	if (argc > optind + 1)
 		usage ();
 
@@ -173,7 +227,6 @@ int main (int argc, char **argv)
 	 * Get the name of the user to check. It is either the command line
 	 * name, or the name getlogin() returns.
 	 */
-
 	if (optind < argc) {
 		user = argv[optind];
 		pw = getpwnam (user);
@@ -198,20 +251,19 @@ int main (int argc, char **argv)
 	/*
 	 * Now we make sure this is a LOCAL password entry for this user ...
 	 */
-
 	if (__ispwNIS ()) {
 		char *nis_domain;
 		char *nis_master;
 
 		fprintf (stderr,
-			 _("%s: cannot change user `%s' on NIS client.\n"),
+			 _("%s: cannot change user '%s' on NIS client.\n"),
 			 Prog, user);
 
 		if (!yp_get_default_domain (&nis_domain) &&
 		    !yp_master (nis_domain, "passwd.byname", &nis_master)) {
 			fprintf (stderr,
 				 _
-				 ("%s: `%s' is the NIS master for this client.\n"),
+				 ("%s: '%s' is the NIS master for this client.\n"),
 				 Prog, nis_master);
 		}
 		exit (1);
@@ -222,7 +274,6 @@ int main (int argc, char **argv)
 	 * Non-privileged users are only allowed to change the shell if the
 	 * UID of the user matches the current real UID.
 	 */
-
 	if (!amroot && pw->pw_uid != getuid ()) {
 		SYSLOG ((LOG_WARN, "can't change shell for `%s'", user));
 		closelog ();
@@ -235,7 +286,6 @@ int main (int argc, char **argv)
 	 * Non-privileged users are only allowed to change the shell if it
 	 * is not a restricted one.
 	 */
-
 	if (!amroot && restricted_shell (pw->pw_shell)) {
 		SYSLOG ((LOG_WARN, "can't change shell for `%s'", user));
 		closelog ();
@@ -248,9 +298,9 @@ int main (int argc, char **argv)
 	 * If the UID of the user does not match the current real UID,
 	 * check if the change is allowed by SELinux policy.
 	 */
-
 	if ((pw->pw_uid != getuid ())
-	    && (checkPasswdAccess (PASSWD__CHSH) != 0)) {
+	    && (is_selinux_enabled () > 0)
+	    && (selinux_check_passwd_access (PASSWD__CHSH) != 0)) {
 		SYSLOG ((LOG_WARN, "can't change shell for `%s'", user));
 		closelog ();
 		fprintf (stderr,
@@ -266,17 +316,45 @@ int main (int argc, char **argv)
 	 * before any changes can be made. Idea from util-linux
 	 * chfn/chsh.  --marekm
 	 */
-
 	if (!amroot && getdef_bool ("CHSH_AUTH"))
 		passwd_check (pw->pw_name, pw->pw_passwd, "chsh");
 
-#endif				/* !USE_PAM */
+#else				/* !USE_PAM */
+	retval = PAM_SUCCESS;
+
+	pampw = getpwuid (getuid ());
+	if (pampw == NULL) {
+		retval = PAM_USER_UNKNOWN;
+	}
+
+	if (retval == PAM_SUCCESS) {
+		retval = pam_start ("chsh", pampw->pw_name, &conv, &pamh);
+	}
+
+	if (retval == PAM_SUCCESS) {
+		retval = pam_authenticate (pamh, 0);
+		if (retval != PAM_SUCCESS) {
+			pam_end (pamh, retval);
+		}
+	}
+
+	if (retval == PAM_SUCCESS) {
+		retval = pam_acct_mgmt (pamh, 0);
+		if (retval != PAM_SUCCESS) {
+			pam_end (pamh, retval);
+		}
+	}
+
+	if (retval != PAM_SUCCESS) {
+		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
+		exit (E_NOPERM);
+	}
+#endif				/* USE_PAM */
 
 	/*
 	 * Now get the login shell. Either get it from the password
 	 * file, or use the value from the command line.
 	 */
-
 	if (!sflg)
 		STRFCPY (loginsh, pw->pw_shell);
 
@@ -284,7 +362,6 @@ int main (int argc, char **argv)
 	 * If the login shell was not set on the command line, let the user
 	 * interactively change it.
 	 */
-
 	if (!sflg) {
 		printf (_("Changing the login shell for %s\n"), user);
 		new_fields ();
@@ -296,7 +373,6 @@ int main (int argc, char **argv)
 	 * users are restricted to using the shells in /etc/shells.
 	 * The shell must be executable by the user.
 	 */
-
 	if (valid_field (loginsh, ":,=")) {
 		fprintf (stderr, _("%s: Invalid entry: %s\n"), Prog, loginsh);
 		closelog ();
@@ -314,7 +390,6 @@ int main (int argc, char **argv)
 	 * to root to protect against unexpected signals. Any
 	 * keyboard signals are set to be ignored.
 	 */
-
 	if (setuid (0)) {
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
 		closelog ();
@@ -327,7 +402,6 @@ int main (int argc, char **argv)
 	 * The passwd entry is now ready to be committed back to
 	 * the password file. Get a lock on the file and open it.
 	 */
-
 	if (!pw_lock ()) {
 		SYSLOG ((LOG_WARN, "can't lock /etc/passwd"));
 		closelog ();
@@ -369,7 +443,6 @@ int main (int argc, char **argv)
 	 * Update the passwd file entry. If there is a DBM file, update
 	 * that entry as well.
 	 */
-
 	if (!pw_update (&pwent)) {
 		SYSLOG ((LOG_ERR, "error updating passwd entry"));
 		closelog ();
@@ -381,7 +454,6 @@ int main (int argc, char **argv)
 	/*
 	 * Changes have all been made, so commit them and unlock the file.
 	 */
-
 	if (!pw_close ()) {
 		SYSLOG ((LOG_ERR, "can't rewrite /etc/passwd"));
 		closelog ();
@@ -399,6 +471,11 @@ int main (int argc, char **argv)
 
 	nscd_flush_cache ("passwd");
 
+#ifdef USE_PAM
+	if (retval == PAM_SUCCESS)
+		pam_end (pamh, PAM_SUCCESS);
+#endif				/* USE_PAM */
+
 	closelog ();
 	exit (E_SUCCESS);
 }
