@@ -29,33 +29,32 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: groupadd.c,v 1.33 2005/06/20 09:57:51 kloczek Exp $")
-#include <sys/types.h>
-#include <stdio.h>
-#include <grp.h>
+#ident "$Id: groupadd.c,v 1.51 2006/01/18 19:55:15 kloczek Exp $"
+
 #include <ctype.h>
 #include <fcntl.h>
-#include "defines.h"
-#include "prototypes.h"
+#include <getopt.h>
+#include <grp.h>
+#include <stdio.h>
+#include <sys/types.h>
+#ifdef USE_PAM
+#include "pam_defs.h"
+#include <pwd.h>
+#endif				/* USE_PAM */
 #include "chkname.h"
+#include "defines.h"
 #include "getdef.h"
 #include "groupio.h"
 #include "nscd.h"
+#include "prototypes.h"
 #ifdef	SHADOWGRP
 #include "sgroupio.h"
-#ifdef USE_PAM
-#include <security/pam_appl.h>
-#include <security/pam_misc.h>
-#include <pwd.h>
-#endif				/* USE_PAM */
 static int is_shadow_grp;
 #endif
 
 /*
  * exit status values
  */
-
 #define E_SUCCESS	0	/* success */
 #define E_USAGE		2	/* invalid command syntax */
 #define E_BAD_ARG	3	/* invalid argument to option */
@@ -63,6 +62,9 @@ static int is_shadow_grp;
 #define E_NAME_IN_USE	9	/* group name not unique */
 #define E_GRP_UPDATE	10	/* can't update group file */
 
+/*
+ * Global variables
+ */
 static char *group_name;
 static gid_t group_id;
 static char *empty_list = NULL;
@@ -83,7 +85,6 @@ static void new_sgent (struct sgrp *);
 static void grp_update (void);
 static void find_new_gid (void);
 static void check_new_name (void);
-static void process_flags (int, char **);
 static void close_files (void);
 static void open_files (void);
 static void fail_exit (int);
@@ -91,10 +92,18 @@ static void fail_exit (int);
 /*
  * usage - display usage message and exit
  */
-
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: groupadd [-g gid [-o]] [-f] group\n"));
+	fprintf (stderr, _("Usage: groupadd [options] group\n"
+			   "\n"
+			   "Options:\n"
+			   "  -f, --force		force exit with success status if the specified\n"
+			   "				group already exists\n"
+			   "  -g, --gid GID		use GID for the new group\n"
+			   "  -h, --help			display this help message and exit\n"
+			   "  -K, --key KEY=VALUE		overrides /etc/login.defs defaults\n"
+			   "  -o, --non-unique		allow create group with duplicate\n"
+			   "				(non-unique) GID\n"));
 	exit (E_USAGE);
 }
 
@@ -104,7 +113,6 @@ static void usage (void)
  *	new_grent() takes all of the values that have been entered and fills
  *	in a (struct group) with them.
  */
-
 static void new_grent (struct group *grent)
 {
 	memzero (grent, sizeof *grent);
@@ -121,7 +129,6 @@ static void new_grent (struct group *grent)
  *	new_sgent() takes all of the values that have been entered and fills
  *	in a (struct sgrp) with them.
  */
-
 static void new_sgent (struct sgrp *sgent)
 {
 	memzero (sgent, sizeof *sgent);
@@ -137,7 +144,6 @@ static void new_sgent (struct sgrp *sgent)
  *
  *	grp_update() writes the new records to the group files.
  */
-
 static void grp_update (void)
 {
 	struct group grp;
@@ -170,6 +176,10 @@ static void grp_update (void)
 		fail_exit (E_GRP_UPDATE);
 	}
 #endif				/* SHADOWGRP */
+#ifdef WITH_AUDIT
+	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group", group_name,
+		      group_id, 1);
+#endif
 	SYSLOG ((LOG_INFO, "new group: name=%s, GID=%u",
 		 group_name, (unsigned int) group_id));
 }
@@ -181,13 +191,12 @@ static void grp_update (void)
  *	file, or checks the given group ID against the existing ones for
  *	uniqueness.
  */
-
 static void find_new_gid (void)
 {
 	const struct group *grp;
 	gid_t gid_min, gid_max;
 
-	gid_min = getdef_unum ("GID_MIN", 100);
+	gid_min = getdef_unum ("GID_MIN", 1000);
 	gid_max = getdef_unum ("GID_MAX", 60000);
 
 	/*
@@ -203,7 +212,6 @@ static void find_new_gid (void)
 	 * user specified one with -g) or looking for the largest unused
 	 * value.
 	 */
-
 #ifdef NO_GETGRENT
 	gr_rewind ();
 	while ((grp = gr_next ())) {
@@ -266,7 +274,6 @@ static void find_new_gid (void)
  *	check_new_name() insures that the new name doesn't contain any
  *	illegal characters.
  */
-
 static void check_new_name (void)
 {
 	if (check_group_name (group_name))
@@ -282,87 +289,12 @@ static void check_new_name (void)
 	exit (E_BAD_ARG);
 }
 
-/*
- * process_flags - perform command line argument setting
- *
- *	process_flags() interprets the command line arguments and sets the
- *	values that the user will be created with accordingly. The values
- *	are checked for sanity.
- */
-
-static void process_flags (int argc, char **argv)
-{
-	char *cp;
-	int arg;
-
-	while ((arg = getopt (argc, argv, "og:O:f")) != EOF) {
-		switch (arg) {
-		case 'g':
-			gflg++;
-			if (!isdigit (optarg[0]))
-				usage ();
-
-			group_id = strtoul (optarg, &cp, 10);
-			if (*cp != '\0') {
-				fprintf (stderr,
-					 _("%s: invalid group %s\n"), Prog,
-					 optarg);
-				fail_exit (E_BAD_ARG);
-			}
-			break;
-		case 'o':
-			oflg++;
-			break;
-		case 'O':
-			/*
-			 * override login.defs defaults (-O name=value)
-			 * example: -O GID_MIN=100 -O GID_MAX=499
-			 * note: -O GID_MIN=10,GID_MAX=499 doesn't work yet
-			 */
-			cp = strchr (optarg, '=');
-			if (!cp) {
-				fprintf (stderr,
-					 _("%s: -O requires NAME=VALUE\n"),
-					 Prog);
-				exit (E_BAD_ARG);
-			}
-			/* terminate name, point to value */
-			*cp++ = '\0';
-			if (putdef_str (optarg, cp) < 0)
-				exit (E_BAD_ARG);
-			break;
-		case 'f':
-			/*
-			 * "force" - do nothing, just exit(0), if the
-			 * specified group already exists. With -g, if
-			 * specified gid already exists, choose another
-			 * (unique) gid (turn off -g). Based on the RedHat's
-			 * patch from shadow-utils-970616-9.
-			 */
-			fflg++;
-			break;
-		default:
-			usage ();
-		}
-	}
-
-	if (oflg && !gflg)
-		usage ();
-
-	if (optind != argc - 1)
-		usage ();
-
-	group_name = argv[argc - 1];
-	check_new_name ();
-}
-
 /*
  * close_files - close all of the files that were opened
  *
  *	close_files() closes all of the files that were opened for this new
  *	group. This causes any modified entries to be written out.
  */
-
 static void close_files (void)
 {
 	if (!gr_close ()) {
@@ -386,15 +318,22 @@ static void close_files (void)
  *
  *	open_files() opens the two group files.
  */
-
 static void open_files (void)
 {
 	if (!gr_lock ()) {
 		fprintf (stderr, _("%s: unable to lock group file\n"), Prog);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "locking group file",
+			      group_name, -1, 0);
+#endif
 		exit (E_GRP_UPDATE);
 	}
 	if (!gr_open (O_RDWR)) {
 		fprintf (stderr, _("%s: unable to open group file\n"), Prog);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "opening group file",
+			      group_name, -1, 0);
+#endif
 		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
@@ -414,24 +353,21 @@ static void open_files (void)
 /*
  * fail_exit - exit with an error code after unlocking files
  */
-
 static void fail_exit (int code)
 {
 	(void) gr_unlock ();
 #ifdef	SHADOWGRP
 	if (is_shadow_grp)
 		sgr_unlock ();
+#endif
+	if (code != E_SUCCESS)
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group",
+			      group_name, -1, 0);
 #endif
 	exit (code);
 }
 
-#ifdef USE_PAM
-static struct pam_conv conv = {
-	misc_conv,
-	NULL
-};
-#endif				/* USE_PAM */
-
 /*
  * main - groupadd command
  */
@@ -444,10 +380,12 @@ int main (int argc, char **argv)
 	int retval;
 #endif
 
+#ifdef WITH_AUDIT
+	audit_help_open ();
+#endif
 	/*
 	 * Get my name so that I can use it to report errors.
 	 */
-
 	Prog = Basename (argv[0]);
 
 	setlocale (LC_ALL, "");
@@ -456,7 +394,88 @@ int main (int argc, char **argv)
 
 	OPENLOG ("groupadd");
 
-	process_flags (argc, argv);
+	{
+		/*
+		 * Parse the command line options.
+		 */
+		char *cp;
+		int option_index = 0;
+		int c;
+		static struct option long_options[] = {
+			{"force", no_argument, NULL, 'f'},
+			{"gid", required_argument, NULL, 'g'},
+			{"help", no_argument, NULL, 'h'},
+			{"key", required_argument, NULL, 'K'},
+			{"non-unique", required_argument, NULL, 'o'},
+			{NULL, 0, NULL, '\0'}
+		};
+
+		while ((c =
+			getopt_long (argc, argv, "fg:hK:o", long_options,
+				     &option_index)) != -1) {
+			switch (c) {
+			case 'f':
+				/*
+				 * "force" - do nothing, just exit(0), if the
+				 * specified group already exists. With -g, if
+				 * specified gid already exists, choose another
+				 * (unique) gid (turn off -g). Based on the RedHat's
+				 * patch from shadow-utils-970616-9.
+				 */
+				fflg++;
+				break;
+			case 'g':
+				gflg++;
+				if (!isdigit (optarg[0]))
+					usage ();
+
+				group_id = strtoul (optarg, &cp, 10);
+				if (*cp != '\0') {
+					fprintf (stderr,
+						 _("%s: invalid group %s\n"),
+						 Prog, optarg);
+					fail_exit (E_BAD_ARG);
+				}
+				break;
+			case 'h':
+				usage ();
+				break;
+			case 'K':
+				/*
+				 * override login.defs defaults (-K name=value)
+				 * example: -K GID_MIN=100 -K GID_MAX=499
+				 * note: -K GID_MIN=10,GID_MAX=499 doesn't work yet
+				 */
+				cp = strchr (optarg, '=');
+				if (!cp) {
+					fprintf (stderr,
+						 _
+						 ("%s: -K requires KEY=VALUE\n"),
+						 Prog);
+					exit (E_BAD_ARG);
+				}
+				/* terminate name, point to value */
+				*cp++ = '\0';
+				if (putdef_str (optarg, cp) < 0)
+					exit (E_BAD_ARG);
+				break;
+			case 'o':
+				oflg++;
+				break;
+			default:
+				usage ();
+			}
+		}
+	}
+
+	if (oflg && !gflg)
+		usage ();
+
+	if (optind != argc - 1)
+		usage ();
+
+	group_name = argv[argc - 1];
+	check_new_name ();
 
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
@@ -488,8 +507,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
 		exit (1);
 	}
-
-	OPENLOG ("groupadd");
 #endif				/* USE_PAM */
 
 #ifdef SHADOWGRP
@@ -499,12 +516,15 @@ int main (int argc, char **argv)
 	/*
 	 * Start with a quick check to see if the group exists.
 	 */
-
 	if (getgrnam (group_name)) {
 		if (fflg) {
 			exit (E_SUCCESS);
 		}
 		fprintf (stderr, _("%s: group %s exists\n"), Prog, group_name);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group",
+			      group_name, -1, 0);
+#endif
 		exit (E_NAME_IN_USE);
 	}
 
@@ -523,18 +543,6 @@ int main (int argc, char **argv)
 	close_files ();
 
 #ifdef USE_PAM
-	if (retval == PAM_SUCCESS) {
-		retval = pam_chauthtok (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
-	}
-
-	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("%s: PAM chauthtok failed\n"), Prog);
-		exit (1);
-	}
-
 	if (retval == PAM_SUCCESS)
 		pam_end (pamh, PAM_SUCCESS);
 #endif				/* USE_PAM */
