@@ -29,17 +29,18 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: grpck.c,v 1.23 2005/04/06 04:26:06 kloczek Exp $")
-#include <stdio.h>
+#ident "$Id: grpck.c,v 1.32 2006/05/07 17:44:39 kloczek Exp $"
+
 #include <fcntl.h>
 #include <grp.h>
-#include "prototypes.h"
-#include "defines.h"
-#include "chkname.h"
 #include <pwd.h>
+#include <stdio.h>
+#include "chkname.h"
 #include "commonio.h"
+#include "defines.h"
 #include "groupio.h"
+#include "nscd.h"
+#include "prototypes.h"
 extern void __gr_del_entry (const struct commonio_entry *);
 extern struct commonio_entry *__gr_get_head (void);
 
@@ -61,9 +62,8 @@ extern struct commonio_entry *__sgr_get_head (void);
 #define	E_CANT_UPDATE	5
 
 /*
- * Local variables
+ * Global variables
  */
-
 static char *Prog;
 static const char *grp_file = GROUP_FILE;
 
@@ -80,7 +80,6 @@ static void delete_member (char **, const char *);
 /*
  * usage - print syntax message and exit
  */
-
 static void usage (void)
 {
 #ifdef	SHADOWGRP
@@ -94,7 +93,6 @@ static void usage (void)
 /*
  * yes_or_no - get answer to question from the user
  */
-
 static int yes_or_no (void)
 {
 	char buf[80];
@@ -102,16 +100,14 @@ static int yes_or_no (void)
 	/*
 	 * In read-only mode all questions are answered "no".
 	 */
-
 	if (read_only) {
-		puts (_("No"));
+		printf (_("No\n"));
 		return 0;
 	}
 
 	/*
 	 * Get a line and see what the first character is.
 	 */
-
 	if (fgets (buf, sizeof buf, stdin))
 		return buf[0] == 'y' || buf[0] == 'Y';
 
@@ -121,7 +117,6 @@ static int yes_or_no (void)
 /*
  * delete_member - delete an entry in a list of members
  */
-
 static void delete_member (char **list, const char *member)
 {
 	int i;
@@ -138,12 +133,11 @@ static void delete_member (char **list, const char *member)
 /*
  * grpck - verify group file integrity
  */
-
 int main (int argc, char **argv)
 {
 	int arg;
 	int errors = 0;
-	int deleted = 0;
+	int changed = 0;
 	int i;
 	struct commonio_entry *gre, *tgre;
 	struct group *grp;
@@ -158,7 +152,6 @@ int main (int argc, char **argv)
 	/*
 	 * Get my name so that I can use it to report errors.
 	 */
-
 	Prog = Basename (argv[0]);
 
 	setlocale (LC_ALL, "");
@@ -170,7 +163,6 @@ int main (int argc, char **argv)
 	/*
 	 * Parse the command line arguments
 	 */
-
 	while ((arg = getopt (argc, argv, "qrs")) != EOF) {
 		switch (arg) {
 		case 'q':
@@ -195,7 +187,6 @@ int main (int argc, char **argv)
 	/*
 	 * Make certain we have the right number of arguments
 	 */
-
 #ifdef	SHADOWGRP
 	if (optind != argc && optind + 1 != argc && optind + 2 != argc)
 #else
@@ -207,7 +198,6 @@ int main (int argc, char **argv)
 	 * If there are two left over filenames, use those as the group and
 	 * group password filenames.
 	 */
-
 	if (optind != argc) {
 		grp_file = argv[optind];
 		gr_name (grp_file);
@@ -224,7 +214,6 @@ int main (int argc, char **argv)
 	/*
 	 * Lock the files if we aren't in "read-only" mode
 	 */
-
 	if (!read_only) {
 		if (!gr_lock ()) {
 			fprintf (stderr, _("%s: cannot lock file %s\n"),
@@ -250,7 +239,6 @@ int main (int argc, char **argv)
 	 * Open the files. Use O_RDONLY if we are in read_only mode,
 	 * O_RDWR otherwise.
 	 */
-
 	if (!gr_open (read_only ? O_RDONLY : O_RDWR)) {
 		fprintf (stderr, _("%s: cannot open file %s\n"), Prog,
 			 grp_file);
@@ -282,7 +270,6 @@ int main (int argc, char **argv)
 	/*
 	 * Loop through the entire group file.
 	 */
-
 	for (gre = __gr_get_head (); gre; gre = gre->next) {
 		/*
 		 * Skip all NIS entries.
@@ -296,22 +283,19 @@ int main (int argc, char **argv)
 		 * have no (struct group) entry because they couldn't be
 		 * parsed properly.
 		 */
-
 		if (!gre->eptr) {
 
 			/*
 			 * Tell the user this entire line is bogus and ask
 			 * them to delete it.
 			 */
-
 			printf (_("invalid group file entry\n"));
-			printf (_("delete line `%s'? "), gre->line);
+			printf (_("delete line '%s'? "), gre->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (!yes_or_no ())
 				continue;
 
@@ -321,11 +305,10 @@ int main (int argc, char **argv)
 			 * When done, it skips back to the top of the loop
 			 * to try out the next list element.
 			 */
-
 		      delete_gr:
 			SYSLOG ((LOG_INFO, "delete group line `%s'",
 				 gre->line));
-			deleted++;
+			changed++;
 
 			__gr_del_entry (gre);
 			continue;
@@ -334,13 +317,11 @@ int main (int argc, char **argv)
 		/*
 		 * Group structure is good, start using it.
 		 */
-
 		grp = gre->eptr;
 
 		/*
 		 * Make sure this entry has a unique name.
 		 */
-
 		for (tgre = __gr_get_head (); tgre; tgre = tgre->next) {
 
 			const struct group *ent = tgre->eptr;
@@ -348,14 +329,12 @@ int main (int argc, char **argv)
 			/*
 			 * Don't check this entry
 			 */
-
 			if (tgre == gre)
 				continue;
 
 			/*
 			 * Don't check invalid entries.
 			 */
-
 			if (!ent)
 				continue;
 
@@ -366,15 +345,13 @@ int main (int argc, char **argv)
 			 * Tell the user this entry is a duplicate of
 			 * another and ask them to delete it.
 			 */
-
-			puts (_("duplicate group entry\n"));
-			printf (_("delete line `%s'? "), gre->line);
+			printf (_("duplicate group entry\n"));
+			printf (_("delete line '%s'? "), gre->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (yes_or_no ())
 				goto delete_gr;
 		}
@@ -384,7 +361,7 @@ int main (int argc, char **argv)
 		 */
 		if (!check_group_name (grp->gr_name)) {
 			errors++;
-			printf (_("invalid group name `%s'\n"), grp->gr_name);
+			printf (_("invalid group name '%s'\n"), grp->gr_name);
 		}
 
 		/*
@@ -392,7 +369,6 @@ int main (int argc, char **argv)
 		 * groups with no members are returned as groups with one
 		 * member "", causing grpck to fail.  --marekm
 		 */
-
 		if (grp->gr_mem[0] && !grp->gr_mem[1]
 		    && *(grp->gr_mem[0]) == '\0')
 			grp->gr_mem[0] = (char *) 0;
@@ -400,7 +376,6 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure each member exists
 		 */
-
 		for (i = 0; grp->gr_mem[i]; i++) {
 			if (getpwnam (grp->gr_mem[i]))
 				continue;
@@ -408,22 +383,94 @@ int main (int argc, char **argv)
 			 * Can't find this user. Remove them
 			 * from the list.
 			 */
-
 			errors++;
 			printf (_("group %s: no user %s\n"),
 				grp->gr_name, grp->gr_mem[i]);
-			printf (_("delete member `%s'? "), grp->gr_mem[i]);
+			printf (_("delete member '%s'? "), grp->gr_mem[i]);
 
 			if (!yes_or_no ())
 				continue;
 
-			SYSLOG ((LOG_INFO, "delete member `%s' group `%s'",
+			SYSLOG ((LOG_INFO, "delete member '%s' group '%s'",
 				 grp->gr_mem[i], grp->gr_name));
-			deleted++;
+			changed++;
 			delete_member (grp->gr_mem, grp->gr_mem[i]);
 			gre->changed = 1;
 			__gr_set_changed ();
 		}
+
+#ifdef	SHADOWGRP
+		/*
+		 * Make sure this entry exists in the /etc/gshadow file.
+		 */
+
+		if (is_shadow) {
+			sgr = (struct sgrp *) sgr_locate (grp->gr_name);
+			if (sgr == NULL) {
+				printf (_
+					("no matching group file entry in %s\n"),
+					sgr_file);
+				printf (_("add group '%s' in %s ?"),
+					grp->gr_name, sgr_file);
+				errors++;
+				if (yes_or_no ()) {
+					struct sgrp sg;
+					struct group gr;
+					static char *empty = NULL;
+
+					sg.sg_name = grp->gr_name;
+					sg.sg_passwd = grp->gr_passwd;
+					sg.sg_adm = &empty;
+					sg.sg_mem = grp->gr_mem;
+					SYSLOG ((LOG_INFO,
+						 "add group `%s' to `%s'",
+						 grp->gr_name, sgr_file));
+					changed++;
+
+					if (!sgr_update (&sg)) {
+						fprintf (stderr,
+							 _
+							 ("%s: can't update shadow entry for %s\n"),
+							 Prog, sg.sg_name);
+						exit (E_CANT_UPDATE);
+					}
+					/* remove password from /etc/group */
+					gr = *grp;
+					gr.gr_passwd = SHADOW_PASSWD_STRING;	/* XXX warning: const */
+					if (!gr_update (&gr)) {
+						fprintf (stderr,
+							 _
+							 ("%s: can't update entry for group %s\n"),
+							 Prog, gr.gr_name);
+						exit (E_CANT_UPDATE);
+					}
+				}
+			} else {
+				/**
+				 * Verify that the all members defined in /etc/group are also
+				 * present in /etc/gshadow.
+				 */
+				char **pgrp_mem, **psgr_mem;
+
+				for (pgrp_mem = grp->gr_mem; *pgrp_mem;
+				     pgrp_mem++) {
+					for (psgr_mem = sgr->sg_mem; *psgr_mem;
+					     psgr_mem++) {
+						if (strcmp
+						    (*pgrp_mem, *psgr_mem) == 0)
+							break;
+					}
+					if (*psgr_mem == NULL) {
+						printf
+						    ("'%s' is a member of the '%s' group in %s but not in %s\n",
+						     *pgrp_mem, sgr->sg_name,
+						     grp_file, sgr_file);
+					}
+				}
+			}
+		}
+#endif
+
 	}
 
 #ifdef	SHADOWGRP
@@ -433,7 +480,6 @@ int main (int argc, char **argv)
 	/*
 	 * Loop through the entire shadow group file.
 	 */
-
 	for (sge = __sgr_get_head (); sge; sge = sge->next) {
 
 		/*
@@ -441,22 +487,19 @@ int main (int argc, char **argv)
 		 * have no (struct sgrp) entry because they couldn't be
 		 * parsed properly.
 		 */
-
 		if (!sge->eptr) {
 
 			/*
 			 * Tell the user this entire line is bogus and ask
 			 * them to delete it.
 			 */
-
 			printf (_("invalid shadow group file entry\n"));
-			printf (_("delete line `%s'? "), sge->line);
+			printf (_("delete line '%s'? "), sge->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (!yes_or_no ())
 				continue;
 
@@ -466,11 +509,10 @@ int main (int argc, char **argv)
 			 * linked list. When done, it skips back to the top
 			 * of the loop to try out the next list element.
 			 */
-
 		      delete_sg:
 			SYSLOG ((LOG_INFO, "delete shadow line `%s'",
 				 sge->line));
-			deleted++;
+			changed++;
 
 			__sgr_del_entry (sge);
 			continue;
@@ -479,13 +521,11 @@ int main (int argc, char **argv)
 		/*
 		 * Shadow group structure is good, start using it.
 		 */
-
 		sgr = sge->eptr;
 
 		/*
 		 * Make sure this entry has a unique name.
 		 */
-
 		for (tsge = __sgr_get_head (); tsge; tsge = tsge->next) {
 
 			const struct sgrp *ent = tsge->eptr;
@@ -493,14 +533,12 @@ int main (int argc, char **argv)
 			/*
 			 * Don't check this entry
 			 */
-
 			if (tsge == sge)
 				continue;
 
 			/*
 			 * Don't check invalid entries.
 			 */
-
 			if (!ent)
 				continue;
 
@@ -511,15 +549,13 @@ int main (int argc, char **argv)
 			 * Tell the user this entry is a duplicate of
 			 * another and ask them to delete it.
 			 */
-
-			puts (_("duplicate shadow group entry\n"));
-			printf (_("delete line `%s'? "), sge->line);
+			printf (_("duplicate shadow group entry\n"));
+			printf (_("delete line '%s'? "), sge->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (yes_or_no ())
 				goto delete_sg;
 		}
@@ -527,19 +563,39 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure this entry exists in the /etc/group file.
 		 */
-
-		if (!gr_locate (sgr->sg_name)) {
-			puts (_("no matching group file entry\n"));
-			printf (_("delete line `%s'? "), sge->line);
+		grp = (struct group *) gr_locate (sgr->sg_name);
+		if (grp == NULL) {
+			printf (_("no matching group file entry in %s\n"),
+				grp_file);
+			printf (_("delete line '%s'? "), sge->line);
 			errors++;
 			if (yes_or_no ())
 				goto delete_sg;
+		} else {
+			/**
+			 * Verify that the all members defined in /etc/gshadow are also
+			 * present in /etc/group.
+			 */
+			char **pgrp_mem, **psgr_mem;
+
+			for (psgr_mem = sgr->sg_mem; *psgr_mem; psgr_mem++) {
+				for (pgrp_mem = grp->gr_mem; *pgrp_mem;
+				     pgrp_mem++) {
+					if (strcmp (*pgrp_mem, *psgr_mem) == 0)
+						break;
+				}
+				if (*pgrp_mem == NULL) {
+					printf
+					    ("'%s' is a member of the '%s' group in %s but not in %s\n",
+					     *psgr_mem, sgr->sg_name, sgr_file,
+					     grp_file);
+				}
+			}
 		}
 
 		/*
 		 * Make sure each administrator exists
 		 */
-
 		for (i = 0; sgr->sg_adm[i]; i++) {
 			if (getpwnam (sgr->sg_adm[i]))
 				continue;
@@ -547,12 +603,11 @@ int main (int argc, char **argv)
 			 * Can't find this user. Remove them
 			 * from the list.
 			 */
-
 			errors++;
 			printf (_
 				("shadow group %s: no administrative user %s\n"),
 				sgr->sg_name, sgr->sg_adm[i]);
-			printf (_("delete administrative member `%s'? "),
+			printf (_("delete administrative member '%s'? "),
 				sgr->sg_adm[i]);
 
 			if (!yes_or_no ())
@@ -561,7 +616,7 @@ int main (int argc, char **argv)
 			SYSLOG ((LOG_INFO,
 				 "delete admin `%s' from shadow group `%s'",
 				 sgr->sg_adm[i], sgr->sg_name));
-			deleted++;
+			changed++;
 			delete_member (sgr->sg_adm, sgr->sg_adm[i]);
 			sge->changed = 1;
 			__sgr_set_changed ();
@@ -570,7 +625,6 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure each member exists
 		 */
-
 		for (i = 0; sgr->sg_mem[i]; i++) {
 			if (getpwnam (sgr->sg_mem[i]))
 				continue;
@@ -578,11 +632,10 @@ int main (int argc, char **argv)
 			/*
 			 * Can't find this user. Remove them from the list.
 			 */
-
 			errors++;
 			printf (_("shadow group %s: no user %s\n"),
 				sgr->sg_name, sgr->sg_mem[i]);
-			printf (_("delete member `%s'? "), sgr->sg_mem[i]);
+			printf (_("delete member '%s'? "), sgr->sg_mem[i]);
 
 			if (!yes_or_no ())
 				continue;
@@ -590,7 +643,7 @@ int main (int argc, char **argv)
 			SYSLOG ((LOG_INFO,
 				 "delete member `%s' from shadow group `%s'",
 				 sgr->sg_mem[i], sgr->sg_name));
-			deleted++;
+			changed++;
 			delete_member (sgr->sg_mem, sgr->sg_mem[i]);
 			sge->changed = 1;
 			__sgr_set_changed ();
@@ -601,11 +654,10 @@ int main (int argc, char **argv)
 #endif				/* SHADOWGRP */
 
 	/*
-	 * All done. If there were no deletions we can just abandon any
+	 * All done. If there were no change we can just abandon any
 	 * changes to the files.
 	 */
-
-	if (deleted) {
+	if (changed) {
 	      write_and_bye:
 		if (!gr_close ()) {
 			fprintf (stderr, _("%s: cannot update file %s\n"),
@@ -624,19 +676,19 @@ int main (int argc, char **argv)
 	/*
 	 * Don't be anti-social - unlock the files when you're done.
 	 */
-
 #ifdef	SHADOWGRP
 	if (is_shadow)
 		sgr_unlock ();
 #endif
 	(void) gr_unlock ();
 
+	nscd_flush_cache ("group");
+
 	/*
 	 * Tell the user what we did and exit.
 	 */
-
 	if (errors)
-		printf (deleted ?
+		printf (changed ?
 			_("%s: the files have been updated\n") :
 			_("%s: no changes\n"), Prog);
 
