@@ -29,10 +29,11 @@
 
 #include <config.h>
 
-#ident "$Id: userdel.c,v 1.58 2005/12/01 20:10:48 kloczek Exp $"
+#ident "$Id: userdel.c,v 1.64 2006/05/12 23:37:33 kloczek Exp $"
 
 #include <errno.h>
 #include <fcntl.h>
+#include <getopt.h>
 #include <grp.h>
 #include <pwd.h>
 #include <stdio.h>
@@ -50,6 +51,7 @@
 #include "pwauth.h"
 #include "pwio.h"
 #include "shadowio.h"
+#include "exitcodes.h"
 #ifdef	SHADOWGRP
 #include "sgroupio.h"
 #endif
@@ -65,9 +67,7 @@
 #define E_HOMEDIR	12	/* can't remove home directory */
 static char *user_name;
 static uid_t user_id;
-static gid_t user_gid;
 static char *user_home;
-static char *user_group;
 
 static char *Prog;
 static int fflg = 0, rflg = 0;
@@ -99,7 +99,12 @@ static void remove_mailbox (void);
  */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: %s [-r] name\n"), Prog);
+	fprintf (stderr, _("Usage: userdel [options] LOGIN\n"
+			   "\n"
+			   "Options:\n"
+			   "  -f, --force			force removal of files, even if not owned by user\n"
+			   "  -h, --help			display this help message and exit\n"
+			   "  -r, --remove			remove home directory and mail spool\n"));
 	exit (E_USAGE);
 }
 
@@ -263,65 +268,6 @@ static void update_groups (void)
 #endif				/* SHADOWGRP */
 }
 
-/*
- * remove_group - remove the user's group unless it is not really a user-private group
- */
-static void remove_group ()
-{
-	char *glist_name;
-	struct group *gr;
-	struct passwd *pwd;
-
-	if (user_group == NULL || user_name == NULL)
-		return;
-
-	if (strcmp (user_name, user_group)) {
-		return;
-	}
-
-	glist_name = NULL;
-	gr = getgrnam (user_group);
-	if (gr)
-		glist_name = *(gr->gr_mem);
-	while (glist_name) {
-		while (glist_name && *glist_name) {
-			if (strncmp (glist_name, user_name, 16)) {
-				return;
-			}
-			glist_name++;
-		}
-	}
-
-	setpwent ();
-	while ((pwd = getpwent ())) {
-		if (strcmp (pwd->pw_name, user_name) == 0)
-			continue;
-
-		if (pwd->pw_gid == user_gid) {
-			return;
-		}
-	}
-
-	/* now actually do the removal if we haven't already returned */
-
-	if (!gr_remove (user_group)) {
-		fprintf (stderr, _("%s: error removing group entry\n"), Prog);
-	}
-#ifdef SHADOWGRP
-
-	/*
-	 * Delete the shadow group entries as well.
-	 */
-
-	if (is_shadow_grp && !sgr_remove (user_group)) {
-		fprintf (stderr, _("%s: error removing shadow group entry\n"),
-			 Prog);
-	}
-#endif				/* SHADOWGRP */
-	SYSLOG ((LOG_INFO, "remove group `%s'\n", user_group));
-	return;
-}
-
 /*
  * close_files - close all of the files that were opened
  *
@@ -384,8 +330,7 @@ static void open_files (void)
 		fprintf (stderr, _("%s: unable to lock password file\n"), Prog);
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "locking password file", user_name, user_id, 1,
-			      0);
+			      "locking password file", user_name, user_id, 0);
 #endif
 		exit (E_PW_UPDATE);
 	}
@@ -572,13 +517,8 @@ static void user_cancel (const char *user)
 	pid = fork ();
 	if (pid == 0) {
 		execl (cmd, cmd, user, (char *) 0);
-		if (errno == ENOENT) {
-			perror (cmd);
-			_exit (127);
-		} else {
-			perror (cmd);
-			_exit (126);
-		}
+		perror (cmd);
+		_exit (errno == ENOENT ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
 	} else if (pid == -1) {
 		perror ("fork");
 		return;
@@ -657,8 +597,6 @@ static void remove_mailbox (void)
 int main (int argc, char **argv)
 {
 	struct passwd *pwd;
-	struct group *grp;
-	int arg;
 	int errors = 0;
 
 #ifdef USE_PAM
@@ -679,16 +617,30 @@ int main (int argc, char **argv)
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
-	while ((arg = getopt (argc, argv, "fr")) != EOF) {
-		switch (arg) {
-		case 'f':	/* force remove even if not owned by user */
-			fflg++;
-			break;
-		case 'r':	/* remove home dir and mailbox */
-			rflg++;
-			break;
-		default:
-			usage ();
+	{
+		/*
+		 * Parse the command line options.
+		 */
+		int c;
+		static struct option long_options[] = {
+			{"force", no_argument, NULL, 'f'},
+			{"help", no_argument, NULL, 'h'},
+			{"remove", no_argument, NULL, 'r'},
+			{NULL, 0, NULL, '\0'}
+		};
+		while ((c =
+			getopt_long (argc, argv, "fhr",
+				     long_options, NULL)) != -1) {
+			switch (c) {
+			case 'f':	/* force remove even if not owned by user */
+				fflg++;
+				break;
+			case 'r':	/* remove home dir and mailbox */
+				rflg++;
+				break;
+			default:
+				usage ();
+			}
 		}
 	}
 
@@ -765,10 +717,6 @@ int main (int argc, char **argv)
 #endif
 	user_id = pwd->pw_uid;
 	user_home = xstrdup (pwd->pw_dir);
-	user_gid = pwd->pw_gid;
-	grp = getgrgid (user_gid);
-	if (grp)
-		user_group = xstrdup (grp->gr_name);
 	/*
 	 * Check to make certain the user isn't logged in.
 	 */
@@ -822,9 +770,6 @@ int main (int argc, char **argv)
 	}
 #endif
 
-	/* Remove the user's group if appropriate. */
-	remove_group ();
-
 	if (rflg) {
 		if (remove_tree (user_home)
 		    || rmdir (user_home)) {
